%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################
 
clear all;
close all;
clc;

global TRUE FALSE mc4rm mc2rm pinfin transient steady M Pcrit

TRUE=1;
FALSE=0;

mc4rm=0;
mc2rm=1;
pinfin=2;

transient=0;
steady=1;


%% INITIALIZING DATA SETS
material_set=[];
tot_source_layers=[];
tot_via_layers=[];
tot_channel_layers=[];
floorplan_set=[];
die_set=[];

%% DEFINING THE PROBLEM- THIS SECTION IS THE ONLY THING TO BE CHANGED IN THIS FILE FOR DIFFERENT PROBLEMS

out_file_name='example_output.txt';

max_iter=25;

stk_2die_example;

htc_value=0;
disp('Enter the correlation you would like to use for HTC. Your options are:');
disp('0- use a fixed value');
disp('1- use Tran et al. (1996)');
disp('2- use Bertsch et al. (2009)');
disp('3- use Kandlikar & Balasubramaniam (2004)');
disp('4- use Three-zone model (Thome et al. 2004)');

correct_value_entered=0;
while (~correct_value_entered)
    htc_choice=input('enter your choice : ');
    if htc_choice==0
        htc_value=input('Enter the fixed value you want to use (in W/um2K): ');
        htc_correlation='fixed';
        correct_value_entered=1;
    elseif htc_choice==1
        htc_correlation='tran';
        correct_value_entered=1;
    elseif htc_choice==2
        htc_correlation='bertsch';
        correct_value_entered=1;
    elseif htc_choice==3
        htc_correlation='kandlikar';
        correct_value_entered=1;
    elseif htc_choice==4
        htc_correlation='threezone';
        correct_value_entered=1;
    else
        disp('Error in input!!! Enter again!');
    end
end


disp('Enter the type of flow for pressure drop calculations. Your options are:');
disp('0- use zero pressure drop');
disp('1- use a homogenous flow model');
disp('2- use a stratified flow model');
correct_value_entered=0;
dpdz_frict_correlation='none';
while (~correct_value_entered)
    flow_type_choice=input('enter your choice : ');
    if flow_type_choice==0
        flow_type='nodrop';
        correct_value_entered=1;
    elseif flow_type_choice==1
        flow_type='homogenous';
        correct_value_entered=1;
    elseif flow_type_choice==2
        flow_type='stratified';
        correct_value_entered=1;
        
        disp('Enter the frictional pressure drop correlation. Your options are:');
        disp('1- use Friedel et al. (1979)');
        disp('2- use Lokhart and Martinelli (1949)');
        disp('3- use Gronnerud et al. (1972)');
        disp('4- use Chisholm et al. (1973)');
        disp('5- use Bankoff et al. (1960)');
        disp('6- use Chawla et al. (1967)');
        disp('7- use Muller et al. (1986)');
        
        correlation_correct_value_entered=0;
        while (~correlation_correct_value_entered)
            dp_choice=input('enter your choice: ');
            if dp_choice==1
                dpdz_frict_correlation='friedel';
                correlation_correct_value_entered=1;
                mu_l=refpropm('V','T',Tin,'Q',0,ref);
                mu_v=refpropm('V','T',Tin,'Q',1,ref);
                if (mu_l/mu_v) >1000
                    disp(['Warning! The ratio of the dynamic viscosities is greater than 1000 (value=' , num2str(mu_l/mu_v) , '). The Friedel correlation might not give accurate results.']);
                end
            elseif dp_choice==2
                dpdz_frict_correlation='martinelli';
                correlation_correct_value_entered=1;
            elseif dp_choice==3
                dpdz_frict_correlation='gronnerud';
                correlation_correct_value_entered=1;
            elseif dp_choice==4
                dpdz_frict_correlation='chisholm';
                correlation_correct_value_entered=1;
            elseif dp_choice==5
                dpdz_frict_correlation='bankoff';
                correlation_correct_value_entered=1;
            elseif dp_choice==6
                dpdz_frict_correlation='chawla';
                correlation_correct_value_entered=1;
            elseif dp_choice==7
                dpdz_frict_correlation='muller';
                correlation_correct_value_entered=1;
            else
                disp('Error in input!!! Enter again!');
            end
            
        end
            
                
    else
        disp('Error in input!!! Enter again!');
    end
end


M=extract_refpropdata(ref,'molecularweight'); %Molar Mass in kg/kmol
    
Pcrit=extract_refpropdata(ref,'criticalpressure'); %Critical pressure in kPa
    
% USE PREVIOUS RESULT , MUST BE STORED IN X0.mat
USE_PREV_RESULT=0; % if 0 then start form t_ambient, if 1 then use results from prev simulation

%% PRINT .stk AND .flp FILES FOR SIMULATION IN THE C VERSION OF 3D-ICE

if PRINT_STK_FLP
    
    print_for_C_version;
    

end

%% VARIABLES FOR SIMULATION


n_layers=length(tot_source_layers);
n_h=n_layers_tot;

n_cells=n_l*n_w*n_h;
n_a=n_l*n_w; %Number of elements in a layer of floorplan

source_dimen=[];
cell_info=[];




%% INITIALIZING SOURCES

for i=1:n_layers
    distribution = floorplan_set(i).name;
    current_source_layer=tot_source_layers(i);
    get_device_parameters;
    
    for j=1:48
        if (d{j}.x >=0 && d{j}.x <=L && d{j}.y>=0 && d{j}.y <=W)
            source_dimen = [source_dimen; d{j}];
        end
    end
end

n_sources = size(source_dimen,1);


%% Displaying the minimum and maximum power density (in W/cm2) based on the first power value 
min_pd=struct('name','temp','pd',0);
max_pd=struct('name','temp','pd',0);
cur_pd=struct('name','temp','pd',0);

total_heat=0;

for count_src=1:n_sources
    pd_cur=source_dimen(count_src).power(1)/(source_dimen(count_src).length*source_dimen(count_src).width*1e-8);
    cur_pd.name=source_dimen(count_src).name;
    cur_pd.pd=pd_cur;
    cur_pd;
    total_heat=total_heat+source_dimen(count_src).power(1);
    
    if count_src==1
        min_pd.name=source_dimen(count_src).name;
        min_pd.pd=pd_cur;
    else
        if min_pd.pd>pd_cur
            min_pd.name=source_dimen(count_src).name;
            min_pd.pd=pd_cur;
        end
    end
    if max_pd.pd<pd_cur
        max_pd.name=source_dimen(count_src).name;
        max_pd.pd=pd_cur;
    end
end

total_heat

min_pd
max_pd

average_pd=total_heat/(L*W*1e-12); % Average power dissipation in SI units

hlv_inlet=refpropm('H','T',Tin,'Q',1,ref)-refpropm('H','T',Tin,'Q',xin,ref);
estimated_cooling=sum(FM_in_set)*n_channel_layers*hlv_inlet;
if (estimated_cooling<=1.1*total_heat)
    disp('Warning: the cooling capacity of the microchannel heat sink not be sufficient for the input heat flux. Danger of Dry out!');
    Total_Heat_Generated=total_heat
    Estimated_Cooling_Capacity=estimated_cooling
    permission=input('Do you wish to continue anyway?(1/0): ');
    
    if ~permission
        simulation_terminated_flag=1;
        disp('Simulation terminated');
        break;
    end
end
        

%% START SIMULATION

if ~USE_PREV_RESULT
    generate_initial_state; 
else
    load X0;
    X=X0;
end

t=0;
time=t;
n_slots=length(source_dimen(1).power);
ntp_final=0;
iter=0;



new_transient_out_node=[];
if ~isempty(transient_out_node)
    T_transient_out_node=[];
    n_out_node=size(transient_out_node,1);
    
    new_transient_out_node=[];
    out_node_cell=[];
    n_new_out_node=0;
    for i=1:n_out_node
            
          
        out_node_cell_cur=(transient_out_node(i,3)-1)*n_a+(transient_out_node(i,2)-1)*n_l+transient_out_node(i,1);
        
        if out_node_cell_cur<1 || out_node_cell_cur>n_cells
            disp('Node meant for observation out of bounds!');
            out_node_cur=[cur_l cur_w cur_h];
            continue;
        else
            new_transient_out_node=[new_transient_out_node; transient_out_node(i,:)];
            out_node_cell=[out_node_cell; out_node_cell_cur];
            n_new_out_node=n_new_out_node+1;
        end
    end
end

%local Mass flow rate- The full extent of the following variables are used
%only in the transient simulation
FM_values=zeros(n_channel_layers*n_channels,n_w);

for count_channel_layer=1:n_channel_layers
    for count_ch=1:n_channels
        count_channel=(count_channel_layer-1)*n_channels+count_ch;
        for count_w=1:n_w
            FM_values(count_channel,count_w)=FM_in_set(count_ch);
        end
    end
end
FM_values_prev_time=FM_values;



% Variables to store local pressures, saturation temperatures and vapor
% qualities in the channel
Psat=Pin*ones(n_channel_layers*n_channels,n_w);
Tsat=Tin*ones(n_channel_layers*n_channels,n_w);
Tsat_prev=Tin*ones(n_channel_layers*n_channels,n_w);
x=xin*ones(n_channel_layers*n_channels,n_w);

% Building cell_info structure and the system matrices
refrigerant_flag=zeros(n_cells,1);
build_cell_info_mc4rm;
build_system_matrices_mc4rm;


tic


if PRINT_MATRICES==TRUE
    print_system_matrix;
end

if analysis_type==transient

    for counter=1:n_slots
        simulation_transient;
        ntp_final_prev=ntp_final;
        [number_cells ntp_slot]=size(X_slot);
        ntp_final=ntp_final+ntp_slot-1;




        %%%%%%%%%%%Transient output calculation for figures%%%%%%%%%%%%%%%%%%%%
        if ~isempty(new_transient_out_node)

            record_data_for_figures;

        end

        %%%%%%%%%%%Transient output calculation for printing in files %%%%%%%%%%%%%%%%%%%%
        if print_output_files

            if counter==1
                init_print_results_in_file;
            end
            print_results_in_file;

        end




        X=X_slot(:,ntp_slot); % initial condition for the next time point

    end
    time=time(1:ntp_final);

else
    
    simulation_steady;
        
        
        
end



        


cput_time_4rm_cttm=toc

final_error=error

if simulation_terminated_flag
    disp('Simulation has been terminated!');
    break;
end




print_results;


clear cell_info;


fclose('all');



min_pd
max_pd


delP_final=Pin-mean(Psat(:,n_w));
rho_l=refpropm('D', 'T', Tin, 'Q', xin, ref);
Vdot_final=700*1e-12*channel_width*channel_height*n_channels/rho_l;
Pumping_power=delP_final*Vdot_final
max_Temp=max(X0)-273.15
heat_in=total_heat
calculate_heat_out

