%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################
function alpha=HTC_from_correlations(x,qs,G,T,Dh,L,ref,htc_correlation,htc_value);

global M Pcrit

%fixed value
if strcmp(htc_correlation,'fixed')
    alpha=20e-9;


% Tran et al (1996) Correlation
elseif strcmp(htc_correlation,'tran')
    hlv=refpropm('H','T',T,'Q',1,ref)-refpropm('H','T',T,'Q',0,ref);
    Bl=qs/(G*hlv); %Boiling number

    sigma_cur=refpropm('I','T',T,'Q',x,ref);
    rho_l=refpropm('D','T',T,'Q',0,ref);
    rho_v=refpropm('D','T',T,'Q',1,ref);
    We_l=G^2*Dh/(rho_l*sigma_cur); %Weber number for the liquid phase

    alpha=8.4e5*(Bl^2*We_l)^(0.3)*(rho_l/rho_v)^(-0.4);
    alpha=alpha*1e-12; % converting back to 3D-ICE units
    
elseif strcmp(htc_correlation,'bertsch')
    
    Rp=90e-3; % Surface roughness in um (CHANGE IF DIFFERENT)
    
    
    Psat=refpropm('P','T',T,'Q',x,ref);
    Preduced=Psat/Pcrit;
    
    
    alpha_NB=55*Preduced^(0.12-0.2*log10(Rp))*(-log10(Preduced))^(-0.55)*M^(-0.5)*qs^(0.67); % Nucleate boiling HTC 
    
    Cp_l=refpropm('C','T',T,'Q',0,ref);
    mu_l=refpropm('V','T',T,'Q',0,ref);
    k_l=refpropm('L','T',T,'Q',0,ref);
    Pr_l=Cp_l*mu_l/k_l;
    Re_l=G*Dh/mu_l;
    alpha_conv_l=(3.66+(0.668*Dh*Re_l*Pr_l/L)/(1+0.04*(Dh*Re_l*Pr_l/L)^(2/3)))*k_l/Dh; % convective HTC in single-phase liquid
    
    Cp_v=refpropm('C','T',T,'Q',1,ref);
    mu_v=refpropm('V','T',T,'Q',1,ref);
    k_v=refpropm('L','T',T,'Q',1,ref);
    Pr_v=Cp_v*mu_v/k_v;
    Re_v=G*Dh/mu_v;
    alpha_conv_v=(3.66+(0.668*Dh*Re_v*Pr_v/L)/(1+0.04*(Dh*Re_v*Pr_v/L)^(2/3)))*k_v/Dh;% convective HTC in single-phase vapor
    
    alpha_conv_tp=(1-x)*alpha_conv_l+x*alpha_conv_v; % convective HTC in two-phase
    
    sigma_cur=refpropm('I','T',T,'Q',x,ref);
    g=9.8;
    rho_l=refpropm('D','T',T,'Q',0,ref);
    rho_v=refpropm('D','T',T,'Q',1,ref);
    Co=sqrt(sigma_cur/(g*(rho_l-rho_v)*Dh^2)); %confinement number
    
    alpha=alpha_NB*(1-x)+alpha_conv_tp*(1+80*(x^2-x^6)*exp(-0.6*Co));
    alpha=alpha*1e-12; % converting back to 3D-ICE units
    
elseif strcmp(htc_correlation,'kandlikar')
    mu_l=refpropm('V','T',T,'Q',0,ref);
    Re_l=G*Dh/mu_l;
    
    k_l=refpropm('L','T',T,'Q',0,ref);
    
    alpha_lo_1600=4.364*k_l/Dh % all-liquid HTC when Re_l<1600
    
    f=(1.58*log(Re_l)-3.28)^(-2);
    Cp_l=refpropm('C','T',T,'Q',0,ref);
    Pr_l=Cp_l*mu_l/k_l;
    alpha_lo_3000=((Re_l-1000)*Pr_l*(f/2)*(k_l/Dh))/(1+12.7*(Pr_l^(2/3)-1)*(f/2)^0.5); % all-liquid HTC when Re_l<1600
    
    if Re_l<1600 % all-liquid HTC
        alpha_lo=alpha_lo_1600;
    elseif Re_l>3000
        alpha_lo=alpha_lo_3000;
    else
        alpha_lo=interp1([1600,3000],[alpha_lo_1600,alpha__lo_3000],Re_l);
    end
    
    rho_l=refpropm('D','T',T,'Q',0,ref);
    rho_v=refpropm('D','T',T,'Q',1,ref);
    
    Co=(rho_v/rho_l)^0.5*((1-x)/x)^0.8; % Convection number
    hlv=refpropm('H','T',T,'Q',1,ref)-refpropm('H','T',T,'Q',0,ref);
    Bo=qs/(G*hlv); % Boiling number
    
    Fsf=0.5; % TEMPORARY VALUE- MUST CHANGE
    
    alpha_NB=(0.6683*Co^(-0.2)*(1-x)^0.8+1058*Bo^0.7*(1-x)^0.8*Fsf)*alpha_lo; % nucleate boiling HTC
    alpha_conv=(1.136*Co^(-0.9)*(1-x)^0.8+667.2*Bo^0.7*(1-x)^0.8*Fsf)*alpha_lo;% convective boiling HTC
    
    [alpha max_val]=max([alpha_NB alpha_conv]);
    if max_val==1
        disp('nucleate!');
    else
        disp('convective!');
    end
    Re_l
    if Re_l<100
        alpha=alpha_NB;
    end
    alpha=alpha*1e-12; % converting back to 3D-ICE units

elseif strcmp(htc_correlation,'threezone')
    rho_l=refpropm('D','T',T,'Q',0,ref);
    rho_v=refpropm('D','T',T,'Q',1,ref);
    
    up=G*(x/rho_v+(1-x)/rho_l); %pair velocity m/s
    Psat=refpropm('P','T',T,'Q',x,ref);
    Preduced=Psat/Pcrit;
    qo=3328*(Preduced)^(-0.5);
    tow=(qs/qo)^(-1.74); % total time period of a triplet
    t_v=tow/(1+(rho_v/rho_l)*((1-x)/x));%time for bubble + vapor slug
    t_l=tow/(1+(rho_l/rho_v)*(x/(1-x)));%time for liquid slug
    
    hlv=refpropm('H','T',T,'Q',1,ref)-refpropm('H','T',T,'Q',0,ref);
    mu_l=refpropm('V','T',T,'Q',0,ref);
    sigma_cur=refpropm('I','T',T,'Q',x,ref);
    Wep=rho_l*Dh*up^2/sigma_cur;
    delta_0=Dh*0.29*(3*(mu_l/(rho_l*up*Dh))^(0.5))^(0.84)*((0.07*Wep^0.41)^(-8)+0.1^(-8))^(-1/8);
    delta_min=90e-9; % Surface roughness in nm
    t_dryfilm=(rho_l*hlv/qs)*(delta_0-delta_min); % time to druness
    
    
    if t_dryfilm>t_v % computation of film time and vapor slug time
        delta_end=delta_0-(qs/(rho_l*hlv))*t_v;
        t_film=t_v;
        t_dry=0;
    else
        delta_end=delta_min;
        t_film=t_dryfilm;
        t_dry=t_v-t_dryfilm;
    end
    
    L_l=up*t_l; %length of liquid slug
    L_v=up*t_v; %length of bubble + vapor slug
    L_dry=up*t_dry; %length of vapor slug
    
    %liquid slug
    Re_l=rho_l*up*Dh/mu_l;
    f_l=(1.58*log(Re_l)-3.28)^(-2);
    Cp_l=refpropm('C','T',T,'Q',0,ref);
    k_l=refpropm('L','T',T,'Q',0,ref);
    Pr_l=Cp_l*mu_l/k_l;
    
    Nu_l_lam=2*0.455*Pr_l^(1/3)*(Re_l*Dh/L_l);
    Nu_l_tran=((f_l/2)*(Re_l-1000)*Pr_l)/(1+12.7*sqrt(f_l/2)*(Pr_l^(2/3)-1))*(1+(Dh/L_l)^(2/3));
    alpha_l=(k_l/Dh)*(Nu_l_lam^4+Nu_l_tran^4)^(1/4);
    
    %vapor slug
    if t_dry
        mu_v=refpropm('V','T',T,'Q',1,ref);
        Re_v=rho_v*up*Dh/mu_v;
        f_v=(1.58*log(Re_v)-3.28)^(-2);
        Cp_v=refpropm('C','T',T,'Q',1,ref);
        k_v=refpropm('L','T',T,'Q',1,ref);
        Pr_v=Cp_v*mu_v/k_v;

        Nu_v_lam=2*0.455*Pr_v^(1/3)*(Re_v*Dh/L_dry);
        Nu_v_tran=((f_v/2)*(Re_v-1000)*Pr_v)/(1+12.7*sqrt(f_v/2)*(Pr_v^(2/3)-1))*(1+(Dh/L_dry)^(2/3));
        alpha_v=(k_l/Dh)*(Nu_v_lam^4+Nu_v_tran^4)^(1/4);
    else
        alpha_v=0;
    end
    
    %film
    alpha_film=(k_l/(delta_0-delta_end))*log(delta_0/delta_end);
    
    alpha=(t_l/tow)*alpha_l+(t_film/tow)*alpha_film+(t_dry/tow)*alpha_v;    
    alpha=alpha*1e-12; % converting back to 3D-ICE units

else 
    disp('Correlation for HTC not found!!');
    return;
    
    
end
