%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################

disp('Step 1: Building cell data');


if channel %FIXING THE WALL MATERIAL
    count_mat_wall=-1;
    for count_mat=1:n_materials
        if strcmp(wall_material,material_set(count_mat).name)
            count_mat_wall=count_mat;
            break;
        end
    end

    k_wall=material_set(count_mat_wall).k;
    sc_wall=material_set(count_mat_wall).sc;

end

if tsv %FIXING THE TSV MATERIAL
    count_mat_tsv=-1;
    for count_mat=1:n_materials
        if strcmp(tsv_material,material_set(count_mat).name)
            count_mat_tsv=count_mat;
            break;
        end
    end

    k_tsv=material_set(count_mat_tsv).k;
    sc_tsv=material_set(count_mat_tsv).sc;

end

%building the cell_info

for count_h=1:n_h 



    if air_heat_sink
        if count_h==n_h || count_h==(n_h-1)
            if count_h==(n_h-1)     %HEAT SPREADER
                for count_w=1:n_w 
                    for count_l=1:n_l 
                        count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l; %Cell we are studying
                        [g_north,g_south,g_east,g_west,g_top,g_bottom,cap]=get_cell_parameters(k_spread_final,sc_spread_final,l_set(count_l),w,h_spread);

                        cell_info(count_cell).north=0;
                        cell_info(count_cell).south=0;
                        cell_info(count_cell).east=0;
                        cell_info(count_cell).west=0;
                        cell_info(count_cell).top=g_top;
                        cell_info(count_cell).bottom=g_bottom;
                        cell_info(count_cell).cap=cap;
                    end
                end
            else                    %HEAT SINK
                for count_w=1:n_w 
                    for count_l=1:n_l 
                        count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l; %Cell we are studying
                        [g_north,g_south,g_east,g_west,g_top,g_bottom,cap]=get_cell_parameters(k_sink_final,sc_sink_final,l_set(count_l),w,h_sink);

                        cell_info(count_cell).north=0;
                        cell_info(count_cell).south=0;
                        cell_info(count_cell).east=0;
                        cell_info(count_cell).west=0;
                        cell_info(count_cell).top=g_top;
                        cell_info(count_cell).bottom=g_bottom;
                        cell_info(count_cell).cap=cap;
                    end
                end
            end

            continue;


        end
    end



    channel_flag=FALSE;

    h=layer_info(count_h).height;
    mat_cur=layer_info(count_h).type;
    count_mat_cur=-1;
    for count_mat=1:n_materials
        if strcmp(mat_cur,material_set(count_mat).name)
            count_mat_cur=count_mat;
            break;
        elseif strcmp(mat_cur,'channel')
            channel_flag=TRUE;
            for temp_mat=1:n_materials
                if strcmp(wall_material,material_set(temp_mat).name)
                    count_mat_cur=temp_mat;
                    break;
                end
            end
            break;

        end
    end




    if tsv
        if find(tot_via_layers==count_h)
            k_cur=material_set(count_mat_cur).k*(1-tsv_ratio)+k_tsv*tsv_ratio;
            sc_cur=material_set(count_mat_cur).sc*(1-tsv_ratio)+sc_tsv*tsv_ratio;
        else
            k_cur=material_set(count_mat_cur).k;
            sc_cur=material_set(count_mat_cur).sc;
        end
    else

        k_cur=material_set(count_mat_cur).k;
        sc_cur=material_set(count_mat_cur).sc;
    end



    if ~channel_flag

        for count_w=1:n_w 
            for count_l=1:n_l 
                count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l; %Cell we are studying
                [g_north,g_south,g_east,g_west,g_top,g_bottom,cap]=get_cell_parameters(k_cur,sc_cur,l_set(count_l),w,h);

                cell_info(count_cell).north=g_north;
                cell_info(count_cell).south=g_south;
                cell_info(count_cell).east=g_east;
                cell_info(count_cell).west=g_west;
                cell_info(count_cell).top=g_top;
                cell_info(count_cell).bottom=g_bottom;
                cell_info(count_cell).cap=cap;


                if count_h==1 %First layer touching the adiabatic PCB
                    cell_info(count_cell).top=inv(inv(g_top)+inv(g_bottom));
                    cell_info(count_cell).bottom=0;

                end

                if ~air_heat_sink

                    if count_h==n_h %Last layer touching the adiabatic Air
                        cell_info(count_cell).bottom=inv(inv(g_top)+inv(g_bottom));
                        cell_info(count_cell).top=0;
                    end
                end


                if count_l==1 %First column touching adiabatic Air
                    cell_info(count_cell).east=inv(inv(g_east)+inv(g_west));
                    cell_info(count_cell).west=0;

                end

                if count_l==n_l %Last column touching adiabatic Air
                    cell_info(count_cell).west=inv(inv(g_east)+inv(g_west));
                    cell_info(count_cell).east=0;
                end

            end
        end

    else %it is channel layer
        for count_w=1:n_w
            for count_l=1:n_l %Count for each row
                count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l; %Cell we are studying

                if rem(count_l,2) %then wall--> since rem returns ZERO if even 
                    k_cur=k_wall;
                    sc_cur=sc_wall;
                    if tsv && find(tot_via_layers==count_h)
                            k_cur=k_wall*(1-tsv_ratio)+k_tsv*tsv_ratio;
                            sc_cur=sc_wall*(1-tsv_ratio)+sc_tsv*tsv_ratio;
                    end
                    [g_north,g_south,g_east,g_west,g_top,g_bottom,cap]=get_cell_parameters(k_cur,sc_cur,l_set(count_l),w,channel_height);
                else % then it is refrigerant
                    refrigerant_flag(count_cell)=1;
                    count_channel_layer=find(tot_channel_layers==count_h);
                    count_channel=(count_channel_layer-1)*n_channels+(count_l/2);
                    build_channel_cell_info;
                end

                cell_info(count_cell).north=g_north;
                cell_info(count_cell).south=g_south;
                cell_info(count_cell).east=g_east;
                cell_info(count_cell).west=g_west;
                cell_info(count_cell).top=g_top;
                cell_info(count_cell).bottom=g_bottom;
                cell_info(count_cell).cap=cap;
                
                %ADIABATIC CONDITIONS- THIS MUST ALWAYS BE WALL
                if count_l==1 %First column touching adiabatic Air
                    cell_info(count_cell).east=inv(inv(g_east)+inv(g_west));
                    cell_info(count_cell).west=0;

                end

                if count_l==n_l %Last column touching adiabatic Air
                    cell_info(count_cell).west=inv(inv(g_east)+inv(g_west));
                    cell_info(count_cell).east=0;
                end


            end
        end
    end









end
