%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################

count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;

%% calculating HTC and pressure drop

% other structural parameters
x_cur=x(count_channel,count_w);
if count_w>1
    x_south=x(count_channel,count_w-1);
else
    x_south=xin;
end

FM_cur=FM_values(count_channel,count_w);
if count_w>1
    FM_south=FM_values(count_channel,count_w-1);
else
    count_ch=count_l/2;
    FM_south=FM_in_set(count_ch);
end

G_cur=FM_cur/(channel_height*channel_width*1e-12); % mass flux in SI units
Dh=2*channel_height*channel_width*1e-6/(channel_height+channel_width); %hydraulic diameter in m
Tsat_cur=Tsat(count_channel,count_w);

 % first calculating heat flux from all four directions
if (analysis_type==steady && iter==0) || (analysis_type==transient && t==0 && iter==0)
    
    qs=average_pd;
    if qs==0
        average_pd=50000; % nominal heat flux of 50W/cm2
    end
    
else
   
    qs=0;
    
    

    %east
    count_neigh=(count_h-1)*n_a+(count_w-1)*n_l+count_l+1;
    g_w=-1*G(count_cell,count_neigh);
    q=g_w*(X(count_neigh)-Tsat_cur);
    if q==0
        q=g_w*10; % a nominal value to begin with
    end
    qs=qs+q;

    %west
    count_neigh=(count_h-1)*n_a+(count_w-1)*n_l+count_l-1;
    g_w=-G(count_cell,count_neigh);
    q=g_w*(X(count_neigh)-Tsat_cur);
    if q==0
        q=g_w*10; % a nominal value to begin with
    end
    qs=qs+q;

    %top
    count_neigh=(count_h)*n_a+(count_w-1)*n_l+count_l;
    g_w=-G(count_cell,count_neigh);
    q=g_w*(X(count_neigh)-Tsat_cur);
    if q==0
        q=g_w*10; % a nominal value to begin with
    end
    qs=qs+q;

    %bottom
    count_neigh=(count_h-2)*n_a+(count_w-1)*n_l+count_l;
    g_w=-G(count_cell,count_neigh);
    q=g_w*(X(count_neigh)-Tsat_cur);
    if q==0
        q=g_w*10; % a nominal value to begin with
    end
    qs=qs+q;
    
    qs=qs/((channel_pitch*w)*1e-12);

end

%calculating pressure drop
[dP_mom dPdz_frict]=Pressuredrop_from_correlations(G_cur,x_cur,x_south,Tsat_cur,Dh,ref,flow_type,dpdz_frict_correlation);
if count_w==1
    Psat_cur=Pin-dP_mom-dPdz_frict*w*1e-6/2;
else
    Psat_cur=Psat(count_channel,count_w-1)-dP_mom-dPdz_frict*w*1e-6;
end

%updating saturation pressure and temperature data
Psat(count_channel,count_w)=Psat_cur;
Tsat_cur=refpropm('T', 'P', Psat_cur, 'Q', x_cur, ref);
Tsat(count_channel,count_w)=Tsat_cur;

% finally calculating HTC based on updated saturation temperature
alpha=HTC_from_correlations(x_cur,qs,G_cur,Tsat_cur,Dh,W,ref,htc_correlation,htc_value);


%% calculating other parameters
if count_w>1
    Tsat_south=Tsat(count_channel,count_w-1);
else
    Tsat_south=Tin;
end
hlv_south=refpropm('H','T',Tsat_south,'Q',1,ref)-refpropm('H','T',Tsat_south,'Q',0,ref);
hlv_cur=refpropm('H','T',Tsat_cur,'Q',1,ref)-refpropm('H','T',Tsat_cur,'Q',0,ref);

rho_g=refpropm('D','T',Tsat_cur,'Q',1,ref);
rho_l=refpropm('D','T',Tsat_cur,'Q',1,ref);
delV=channel_width*channel_height*w;
C_hat=hlv_cur*delV*((rho_g^2*rho_l)/(rho_l*x_cur+rho_g*(1-x_cur))^2);

%% plugging into the cell structure

g_east=alpha*w*channel_height;
g_west=g_east;
g_top=alpha*w*l_set(count_l);
g_bottom=g_top;
g_north=FM_cur*hlv_cur;
g_south=FM_south*hlv_south;
cap=C_hat;