%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################

disp('Step 2: begin construction of MNA matrices');



G=spalloc(n_cells,n_cells,n_cells);
C=spalloc(n_cells,n_cells,n_cells);
U=zeros(n_cells,1);
Utop=zeros(n_cells,1);
Ubottom=zeros(n_cells,1);
Ueast=zeros(n_cells,1);
Uwest=zeros(n_cells,1);

for count_h=1:n_h

    for count_w=1:n_w
        for count_l=1:n_l

            count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;

            if ~refrigerant_flag(count_cell) %its not refrigerant


                g_cur=0;

                %east
                count_neigh=(count_h-1)*n_a+(count_w-1)*n_l+count_l+1;
                if count_l<n_l && cell_info(count_cell).east~=0 && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).east)+inv(cell_info(count_neigh).west));
                    G(count_cell,count_cell+1)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;
                end
%                             

                %west
                count_neigh=(count_h-1)*n_a+(count_w-1)*n_l+count_l-1;
                if count_l>1 && cell_info(count_cell).west~=0 && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).west)+inv(cell_info(count_neigh).east));
                    G(count_cell,count_cell-1)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;
                 end
%                             

                %north
                count_neigh=(count_h-1)*n_a+(count_w)*n_l+count_l;
                if count_w<n_w && cell_info(count_cell).north~=0 && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).north)+inv(cell_info(count_neigh).south));
                    G(count_cell,count_cell+n_l)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;
                end
%                         
%                             

                %south
                count_neigh=(count_h-1)*n_a+(count_w-2)*n_l+count_l;
                if count_w>1 && cell_info(count_cell).south~=0 && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).south)+inv(cell_info(count_neigh).north));
                    G(count_cell,count_cell-n_l)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;
                end
%                            


                %top
                count_neigh=(count_h)*n_a+(count_w-1)*n_l+count_l;
                if count_h<n_h && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).top)+inv(cell_info(count_neigh).bottom));
                    G(count_cell,count_cell+n_a)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;                            
                 
                    
                elseif air_heat_sink
                    if zero_air_resistance_flag
                        eff_conductance=cell_info(count_cell).top;
                    else
                        eff_conductance=inv(inv(cell_info(count_cell).top)+inv(get_interface(H_air_final,l_set(count_l),w)));
                    end
                    g_cur=g_cur+eff_conductance;
                    Utop(count_cell)=Utop(count_cell)+eff_conductance*Tamb;
                end

                %bottom
                count_neigh=(count_h-2)*n_a+(count_w-1)*n_l+count_l;
                if count_h>1 && ~refrigerant_flag(count_neigh)
                    eff_conductance=inv(inv(cell_info(count_cell).bottom)+inv(cell_info(count_neigh).top));
                    G(count_cell,count_cell-n_a)=-1*eff_conductance;
                    g_cur=g_cur+eff_conductance;

                 end


                
                G(count_cell,count_cell)=G(count_cell,count_cell)+g_cur;
                C(count_cell,count_cell)=cell_info(count_cell).cap;

            else %its refrigerant


                count_channel_layer=find(tot_channel_layers==count_h);
                count_channel=(count_channel_layer-1)*n_channels+(count_l/2);
                build_channel_system_matrices_mc4rm;


            end






        end
    end
end
if analysis_type==transient
    A=G+C*(1/h_step);
end






%Collect all the information of the floorplan

%FIXING SOURCES


% disp('begin construction of source vector');

disp('Step 3: Fixing source locations');

for src_count=1:n_sources


    src_left=source_dimen(src_count).x; %X coordinate
    src_right=src_left+source_dimen(src_count).length;
    src_bottom=source_dimen(src_count).y; %Y coordinate
    src_top=src_bottom+source_dimen(src_count).width; 


    count_h=source_dimen(src_count).layer; %Layer in which the source is

    src_l_dump=[];
    src_w_dump=[];



    %For every cell in the layer
    for count_w=1:n_w
        for count_l=1:n_l
            count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;

            %Location of the cell mid point
            cell_x=sum(l_set(1:count_l))-l_set(count_l)/2;
            cell_y=count_w*w-w/2;

            %If we find a heater...
            if ( (cell_x>=src_left) && (cell_x<src_right) && (cell_y>=src_bottom) && (cell_y<src_top))
                cell_info(count_cell).src_number=src_count;

                src_l_dump=[src_l_dump count_l];
                src_w_dump=[src_w_dump count_w];



%                     [count_l count_w count_h]
            end
        end
    end

    nsrc=length(src_l_dump);
    source_factor=zeros(1,nsrc);
    src_area=0;

    for count_src=1:nsrc
        source_factor(count_src)=l_set(src_l_dump(count_src))*w;
        src_area=src_area+source_factor(count_src);
    end

    source_factor=source_factor/src_area;

    for count_src=1:nsrc
        count_l=src_l_dump(count_src);
        count_w=src_w_dump(count_src);
        count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;

        cell_info(count_cell).src_factor=source_factor(count_src);
    end

end
