%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################

%%  FIXING SINK MATERIAL
count_mat_sink=-1;
for count_mat=1:n_materials
    if strcmp(tsv_material,material_set(count_mat).name)
        count_mat_sink=count_mat;
        break;
    end
end
if count_mat_sink==-1
    disp('Sink material not found');
    sink_material
    simulation_terminated_flag=1;
    break;
end
k_sink=material_set(count_mat_sink).k;
sc_sink=material_set(count_mat_sink).sc;

%% FIXING SPREAD MATERIAL 
count_mat_spread=-1;
for count_mat=1:n_materials
    if strcmp(tsv_material,material_set(count_mat).name)
        count_mat_spread=count_mat;
        break;
    end
end
if count_mat_spread==-1
    disp('Spread material not found');
    spread_material
    simulation_terminated_flag=1;
    break;
end
k_spread=material_set(count_mat_spread).k;
sc_spread=material_set(count_mat_spread).sc;


%% AREAS
A_sink=L_sink*W_sink;
A_spread=L_spread*W_spread;
A_die=L*W;


%% CALCULATION FOR SINK
gm_sink_total=k_sink*A_sink/h_sink;

e_sink=sqrt(A_spread/A_sink);
Bi_sink=H_air*sqrt(A_sink/pi)/k_sink;
t_sink=h_sink/sqrt(A_sink/pi);
lambdac_sink=pi+1/(sqrt(pi)*e_sink);
phic_sink=(tanh(lambdac_sink*t_sink)+(lambdac_sink/Bi_sink))/(1+(lambdac_sink/Bi_sink)*tanh(lambdac_sink*t_sink));
gcon_sink_total=(2*k_sink*sqrt(A_spread))/((1-e_sink)^1.5*phic_sink);


g_sink_total=(gcon_sink_total*gm_sink_total)/(gcon_sink_total+gm_sink_total);

k_sink_final=g_sink_total*h_sink/A_die;
sc_sink_final=sc_sink*A_sink/A_die;


%% calculation of H_sink --> used for spread

R_air=1/(H_air*A_sink);
R_sink=1/g_sink_total;
H_sink=(1/(R_air+R_sink))*(1/A_spread);

%% CALCULATION FOR SPREAD
gm_spread_total=k_spread*A_spread/h_spread;

e_spread=sqrt(A_die/A_spread);
Bi_spread=H_sink*sqrt(A_spread/pi)/k_spread;
t_spread=h_spread/sqrt(A_spread/pi);
lambdac_spread=pi+1/(sqrt(pi)*e_spread);
phic_spread=(tanh(lambdac_spread*t_spread)+(lambdac_spread/Bi_spread))/(1+(lambdac_spread/Bi_spread)*tanh(lambdac_spread*t_spread));
gcon_spread_total=(2*k_spread*sqrt(A_die))/((1-e_spread)^1.5*phic_spread);


g_spread_total=(gcon_spread_total*gm_spread_total)/(gcon_spread_total+gm_spread_total);

k_spread_final=g_spread_total*h_spread/A_die;
sc_spread_final=sc_spread*A_spread/A_die;

%% CALCULATION OF H_air_final

H_air_final=H_air*A_sink/A_die;
