%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################
stk=fopen(stack_file_name,'w');

for i=1:n_materials
    fprintf(stk,'material %s :\n',material_set(i).name);
    fprintf(stk,'\t thermal conductivity %d ;\n',material_set(i).k);
    fprintf(stk,'\t volumetric heat capacity %d ;\n\n',material_set(i).sc);
end

fprintf(stk,'\n');

if air_heat_sink
    fprintf(stk, 'conventional heat sink: \n');
    fprintf(stk,'\t heat transfer coefficient %d ;\n',H_air_final);
    fprintf(stk,'\t ambient temperature %d ;\n', t_ambient);
end

if channel
    fprintf(stk,'channel: \n\n');
    
    fprintf(stk,'\t height %d ;\n\n',channel_height);
    
    fprintf(stk,'\t channel length %d ;\n',channel_width);
    fprintf(stk,'\t wall length %d ;\n\n',channel_pitch-channel_width);
    
    fprintf(stk,'\t first wall length %d ;\n',first_wall);
    fprintf(stk,'\t last wall length %d ;\n\n',first_wall);
    
    fprintf(stk,'\t wall material %s ;\n\n',wall_material);
    
    fprintf(stk,'\t coolant flow rate %d ;\n',round(L/channel_pitch)*channel_height*channel_width*60/1e12);
    fprintf(stk,'\t heat transfer coefficient side %d ;\n',k_channel_side);
    fprintf(stk,'\t \t \t \t top %d ;\n',k_channel_top);
    fprintf(stk,'\t \t \t \t bottom %d ;\n',k_channel_bottom);
    fprintf(stk,'\t coolant volumetric heat capacity %d ;\n',sc_water);
    fprintf(stk,'\t coolant incoming temperature %d ;\n\n',coolant_temp);
    
    
end

fprintf(stk,'\n');

n_layers=length(tot_source_layers);

die_object='die';

for i=1:n_layers
    die_object_cur=horzcat(die_object,int2str(i));
    fprintf(stk,'die %s:\n',die_object_cur);
    fprintf(stk,'\t source %d %s ;\n\n',layer_info(tot_source_layers(i)).height,layer_info(tot_source_layers(i)).type);
    
end


fprintf(stk,'stack :\n');
for i=n_layers_tot:-1:1
    if air_heat_sink
        if i==n_layers_tot || i==n_layers_tot-1
            continue;
        end
    end
    
    if find(i==tot_source_layers)
        count_die=find(i==tot_source_layers);
        die_object_cur=horzcat(die_object,int2str(count_die));
        floorplan_object_cur=horzcat(floorplan_set(count_die).name,'.flp');
        die_instance_cur=die_set(count_die).name;
        fprintf(stk,'\t die %s %s floorplan "%s" ;\n',die_instance_cur,die_object_cur,floorplan_object_cur);
    else
        layer_instance_cur=horzcat(layer_info(i).type,int2str(i));
        fprintf(stk,'\t layer %s %d %s ;\n',layer_instance_cur,layer_info(i).height,layer_info(i).type);
    end
end

fprintf(stk,'\n');

fprintf(stk,'dimensions :\n');
fprintf(stk,'\t chip length %d ;\n',L);
fprintf(stk,'\t chip width %d ;\n\n',W);

fprintf(stk,'\t cell width %d ;\n',l);
fprintf(stk,'\t cell width %d ;\n\n',w);

fclose(stk);



for i=1:n_layers
    distribution = floorplan_set(i).name;
    current_source_layer=tot_source_layers(i);
    get_device_parameters;

    aless_filename=horzcat(distribution,'.flp');
    aless=fopen(aless_filename,'w');


    for j=1:48
        if (d{j}.x >=0 && d{j}.x <=L && d{j}.y>=0 && d{j}.y <=W)
            fprintf(aless,'%s: \n',d{j}.name);
            fprintf(aless,'position %d, %d ;\n',d{j}.x,d{j}.y);
            fprintf(aless,'dimension %d, %d ;\n',d{j}.length,d{j}.width);
            fprintf(aless,'power ');
            for count=1:length(d{j}.power)
                fprintf(aless,'%d ',d{j}.power(count));
            end


            fprintf(aless,';\n \n');
        end
    end
    fclose(aless);
end

fclose('all');