%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################
%% PRINTING TRANSIENT RESPONSE
if (analysis_type==transient && ~isempty(new_transient_out_node))
    
    title_transient='Transient Response at the node [';
    for i=1:n_new_out_node
        
        figure;
        title_cur=horzcat(title_transient,int2str(new_transient_out_node(i,1)),',',int2str(new_transient_out_node(i,2)),',',int2str(new_transient_out_node(i,3)),']');
        plot(time,T_transient_out_node(i,:));
        xlabel('Time (s)');
        ylabel('Temperature (K)');
        title(title_cur);
        
    end
end

%% PRINTING SNAPSHOTS
if ~isempty(snapshot_die) || ~isempty(snapshot_layer) || ~isempty(vertical_snapshot)

    n_hot_spots=[];
    
    final_min=min(X);
    final_max=max(X);
    
    x_axis=cumsum([0 l_set]);
    y_axis=(0:1:n_w)*w;
    z_temp=0;
    if ~air_heat_sink
        for count_h=1:n_h
            z_temp=[z_temp; layer_info(count_h).height];
        end
    else
        for count_h=1:(n_h-2)
            z_temp=[z_temp; layer_info(count_h).height];
        end
        z_temp=[z_temp;h_spread;h_sink];
    end

    z_axis=cumsum(z_temp)';




    area_map=zeros(n_w+1,n_l+1,n_h);
    for count_h=1:n_h
        temporal = 0;
        for count_w=1:(n_w+1)
            for count_l=1:(n_l+1)
                if count_l == (n_l+1)
                    area_map(count_w,count_l,count_h)= area_map(count_w,count_l-1,count_h);
                    elseif count_w == (n_w+1)
                        area_map(count_w,count_l,count_h)= area_map(count_w-1,count_l,count_h);
                    elseif count_l == (n_l+1) & count_w == (n_w+1)
                        area_map(count_w,count_l,count_h)= area_map(count_w-1,count_l-1,count_h);
                else            
                count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                area_map(count_w,count_l,count_h)=X(count_cell);
                    if(X(count_cell) > 317)
                        temporal=temporal+1;
                    end
                end
            end
        end
        n_hot_spots=[n_hot_spots, temporal];
    end
    
    
    %% PRINGINT DIE SNAPSHOT
    if ~isempty(snapshot_die)
        dies_min=[];
        dies_max=[];
        for j=1:n_layers
            dies_min=[dies_min min(min(area_map(:,:,tot_source_layers(j))))];
            dies_max=[dies_max max(max(area_map(:,:,tot_source_layers(j))))];
        end
        
        n_snapshot_die=size(snapshot_die,1);
        title_snapshot_die='Snapshot for the die ';
        for j=1:n_snapshot_die
            die_cur=snapshot_die(j).name;
            i=-1;
            for k=1:n_layers
                if strcmp(die_cur,die_set(k).name);
                    i=k;
                    break;
                end
            end
            if i==-1
                disp('Die meant for snapshot not found!');
                die_cur
                continue;
            end
            
            die_min=dies_min(i); 
            die_max=dies_max(i);
            
            title_cur=horzcat(title_snapshot_die, die_cur);
            figure;
            h=surf(x_axis,y_axis,area_map(:,:,tot_source_layers(i))-273.15);
            set(h,'edgecolor','none');
            colormap(flipud(hot));
            colorbar;
            caxis([min(dies_min) max(dies_max)]-273.15);
            view([0 90]);
            axis equal; 
            paint_floorplan;
            xlabel('length (um)');
            ylabel('width (um)');
            zlabel('Temperature (K)');
            title(title_cur);
        end
    end
    
    %% PRINTING LAYER SNAPSHOT
    if ~isempty(snapshot_layer)
        n_snapshot_layer=size(snapshot_layer,1);
        title_snapshot_layer='Snapshot for the layer ';
        
        for j=1:n_snapshot_layer
            i=snapshot_layer(j);
            if i>n_h || i<1
                disp('Layer meant for snapshot is out of bounds!');
                snapshot_layer_number=i
                continue;
            end
            
            title_cur=horzcat(title_snapshot_layer, int2str(i));
            figure;
            h=surf(x_axis,y_axis,area_map(:,:,i));
            set(h,'edgecolor','none');
            colormap(flipud(hot));
            colorbar;
            caxis([final_min final_max]);
            view([0 90]);
            axis equal; 
            xlabel('length (um)');
            ylabel('width (um)');
            zlabel('Temperature (K)');
            title(title_cur);
        end
    end
    
    %% PRINGINT VERTICAL SNAPSHOT
    
    if ~isempty(vertical_snapshot)
        n_vertical_snapshot=size(vertical_snapshot,1);
        title_vertical_snapshot='Vertical Snapshot at the column#=';
        
        for j=1:n_vertical_snapshot

            vertical_map=zeros(n_h+1,n_w+1);
            l_cur=vertical_snapshot(j);
            
            if l_cur<1 || l_cur>n_l
                disp('Col # meant for vertical snapshot out of bounds!');
                vertical_snapshot_l=l_cur
                continue;
            end

            for count_h=1:(n_h+1)
                for count_w=1:(n_w+1)
                    if count_w == (n_w+1)
                        vertical_map(count_h,count_w)= vertical_map(count_h,count_w-1);
                    elseif count_h == (n_h+1)
                        vertical_map(count_h,count_w)= vertical_map(count_h-1,count_w);
                    elseif count_w == (n_w+1) & count_h == (n_h+1)
                        vertical_map(count_h,count_w)= vertical_map(count_h-1,count_w-1);
                    else            
                        count_cell=(count_h-1)*n_a+(count_w-1)*n_l+l_cur;
                        vertical_map(count_h,count_w)=X(count_cell);
                    end
                end
            end
            
            title_cur=horzcat(title_vertical_snapshot,int2str(l_cur));
            
            figure;
            if air_heat_sink
                h=pcolor(y_axis,z_axis(1:n_h-2),vertical_map(1:n_h-2,:));
            else
                h=pcolor(y_axis,z_axis,vertical_map);
            end
            set(h,'edgecolor','none');
            colorbar;
            caxis([final_min final_max]);
            view([0 90]);
            xlabel('width (um)');
            ylabel('height (um)');
            zlabel('Temperature (K)');
            title(title_cur);
        end

    end

end

%% PRINGINT DIE POWERMAP
if ~isempty(powermap_die)
    x_axis=cumsum([0 l_set]);
    y_axis=(0:1:n_w)*w;
    die_min=0;
    die_max=max_pd.pd;
    
    n_powermap_die=size(powermap_die,1);
    title_powermap_die='Power map for the die ';
    for j=1:n_powermap_die
        die_cur=powermap_die(j).name;
        i=-1;
        for k=1:n_layers
            if strcmp(die_cur,die_set(k).name);
                i=k;
                break;
            end
        end
        if i==-1
            disp('Die meant for powermap not found!');
            die_cur
            continue;
        end
        power_map=zeros(n_w+1,n_l+1);
        count_h=i;
        for count_w=1:(n_w+1)
            for count_l=1:(n_l+1)
                if count_l == (n_l+1)
                    power_map(count_w,count_l)= power_map(count_w,count_l-1);
                    elseif count_w == (n_w+1)
                        power_map(count_w,count_l)= power_map(count_w-1,count_l);
                    elseif count_l == (n_l+1) & count_w == (n_w+1)
                        power_map(count_w,count_l)= power_map(count_w-1,count_l-1);
                else            
                    count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                    src_count=cell_info(count_cell).src_number;
                    if src_count~=0
                        source_factor=cell_info(count_cell).src_factor;
                        amp=source_dimen(src_count).power(1)*source_factor*1e8/(l_set(count_l)*w);
                    else
                        amp=0;
                    end
                    power_map(count_w,count_l,count_h)=amp;
                    
                end
            end
        end
        


        title_cur=horzcat(title_powermap_die, die_cur);
        figure;
        h=surf(x_axis,y_axis,power_map);
        set(h,'edgecolor','none');
%         colormap(flipud(hot));
        colorbar;
        view([0 90]);
%         axis equal; 
        paint_floorplan;
        xlabel('length (um)');
        ylabel('width (um)');
        zlabel('Heat flux (W/cm2)');
        title(title_cur);
    end
end
