%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################
% X0 --> known solution from previous time point
% X --> solution from the previous iteration in the current time point
% X1 --> solution from the current iteration in the current time point
% X_slot --> stores solutions from current time slot

X_slot=X;
t_temp=0;
time_temp=[];

last_time=time(length(time));

simulation_terminated_flag=0;




while (t_temp<slot_time)%(steady_state_error>1e-3)

    X0=X;
    FM_values_prev_time=FM_values;
    
    t_temp=t_temp+h_step;
    
    fcount=1;
        
    
    clear source_factor;
    
    %%% PLUGGING SOURCE VALUES AT EACH TIME STEP
        
    Ua=zeros(n_cells,1);
    time_step=last_time+t_temp

    for layer_count=1:n_layers
        count_h=tot_source_layers(layer_count);

        for count_w=1:n_w
            for count_l=1:n_l
                count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                
                src_count=cell_info(count_cell).src_number;
                if src_count~=0


                    source_factor=cell_info(count_cell).src_factor;

                    amp=source_dimen(src_count).power(counter)*source_factor;

                    amp_temp=amp;

                    Ua(count_cell)=Ua(count_cell)+input_ramp_function(amp_temp,t_temp,input_rise_time);
                end
            end
        end


    end
    
    % BEGINNING OF ITERATIONS FOR EACH TIME STEP
    
    
    error=100;
    error_change_percentage=100;

    iter=0;    
    iter_set=[];
    error_steady=[];


    while error>error_tolerance && iter<=max_iter && error_change_percentage>1
        iter=iter+1
        steady_put_sources;
        X1=solve_with_lu(G+C/h_step,C*X0/h_step+U+Utop+Ubottom+Ueast+Uwest+Ua);
        
        error=norm(abs(X1-X))/norm(X1);
        if counter==1 && iter==1 % exception for ploting
            iter_set=iter;
            error_steady=error;
            figure(1);
            error_h=semilogy(iter_set,error_steady);
            set(error_h,'XDataSource','iter_set');
            set(error_h,'YDataSource','error_steady');
            xlabel('Iteration Number');
            ylabel('Error Norm');
            title(horzcat(['Error Norm Evolution for time step ' num2str(last_time+t_temp)]));
            pause(1);

        else
            error_change_percentage=abs(error-error_steady(length(error_steady)))*100/error;
            iter_set=[iter_set iter];
            error_steady=[error_steady error];

            refreshdata(error_h);
            title(horzcat(['Error Norm Evolution for time step ' num2str(last_time+t_temp)]));
            pause(1);
        end
        
        X=X1;
        

        check_vapor_quality;
        if simulation_terminated_flag
            calculate_heat_out;
            heat_in=total_heat
            heat_out
            plot_results_for_debugging;
            break;
        end

        Psat_prev=Psat;
        Tsat_prev=Tsat;
        
        % Updating FM_values
        
        for count_channel_layer=1:n_channel_layers
            count_h=tot_channel_layers(count_channel_layer);
            for count_l=2:2:(n_l-1)
                count_channel=(count_channel_layer-1)*n_channels+(count_l/2);
                for count_w=1:n_w
                    count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                    calculate_FM;
                end
            end
        end


        % Updating channel parameters
        for count_channel_layer=1:n_channel_layers
            count_h=tot_channel_layers(count_channel_layer);
            for count_l=2:2:(n_l-1)
                count_channel=(count_channel_layer-1)*n_channels+(count_l/2);
                for count_w=1:n_w
                   count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                   build_channel_cell_info;
                   cell_info(count_cell).north=g_north;
                   cell_info(count_cell).south=g_south;
                   cell_info(count_cell).east=g_east;
                   cell_info(count_cell).west=g_west;
                   cell_info(count_cell).top=g_top;
                   cell_info(count_cell).bottom=g_bottom;
                   cell_info(count_cell).cap=cap;
                end
            end
        end
        for count_channel_layer=1:n_channel_layers
            count_h=tot_channel_layers(count_channel_layer);
            for count_l=2:2:(n_l-1)
                count_channel=(count_channel_layer-1)*n_channels+(count_l/2);
                for count_w=1:n_w
                   count_cell=(count_h-1)*n_a+(count_w-1)*n_l+count_l;
                   build_channel_system_matrices_mc4rm;
                end
            end
        end
        
        
        
    end        


    X_slot=[X_slot X];    
    time_temp=[time_temp t_temp];
    
    
    
end

if simulation_terminated_flag
    break;
end
                

time_temp=time_temp+last_time;

time=[time time_temp];
ntp=length(time);
t=time(ntp);
