%  ##############################################################################
%  # This file is part of STEAM, version 1.0 (pre-alpha) .                      #
%  #                                                                            #
%  # STEAM is free software: you can  redistribute it and/or  modify it  under  #
%  # the terms of the  Ecliplse Publice License as  published by the            #
%  # Eclipse  Foundation, version 1.0.                                          #
%  #                                                                            #
%  # STEAM is  distributed  in the hope  that it will  be useful, but  WITHOUT  #
%  # ANY  WARRANTY; without  even the  implied warranty  of MERCHANTABILITY  or #
%  # FITNESS  FOR A PARTICULAR  PURPOSE. See the Eclipse Public License for     #
%  # more details. Visit <http://www.eclipse.org/legal/epl-v10.html>.           #
%  #                                                                            #
%  #                             Copyright (C) 2014                             #
%  #   Embedded Systems Laboratory - Ecole Polytechnique Federale de Lausanne   #
%  #                            All Rights Reserved.                            #
%  #                                                                            #
%  # Authors: Arvind Sridhar                                                    #
%  #          David Atienza                                                     #
%  #                                                                            #
%  # For any comment, suggestion or request  about STEAM, please  contact       #
%  #                                                                            #
%  # EPFL-STI-IEL-ESL                                                           #
%  # Batiment ELG, ELG 130                                                      #
%  # Station 11                                                                 #
%  # 1015 Lausanne, Switzerland           Url  : http://esl.epfl.ch/STEAM       #
%  ##############################################################################

% THIS IS THE STACK DESCRIPTION FILE. ALL DIMENSIONS IN um


get_material_parameters;




%% DIMENSIONS  OF THE CHIP
L=10000; % 92
W=10000;

%% CONVENTIONAL AIR HEAT SINK
air_heat_sink=FALSE;

H_air=10/36e8; % FROM HOTSPOT- 0.1K/W FOR A 6cmX6cm CHIP

L_sink=10e4;
W_sink=10e4;
h_sink=2e4;
sink_material='copper';

L_spread=3e4;
W_spread=3e4;
h_spread=1000;
spread_material='copper';


zero_air_resistance_flag=0; % assign this value to be 1 if you want zero silicon to air resistance, otherwise, make it 0 
Tamb=300;



%% TSVS
tsv=FALSE;
tsv_area=10*10; % crosssection of a single tsv , in um^2
n_tsv=512*16; %total number of tsvs in a given layer
tsv_material='copper';


%% CHANNEL PARAMETERS- ALWAYS ASSUMED TO BE FACING 'NORTH'

channel=TRUE;
channel_type=mc4rm;

ref='r245fa.fld'; 
Inlet_sat_temperature=31.1; %inlet saturation pressure bar
xin=0; % input vapor quality (no units)

Tin=Inlet_sat_temperature+273.15;   

Pin=refpropm('P', 'T', Tin, 'Q', xin, ref);


channel_width=50;
channel_pitch=100;
channel_height=200;
wall_material='silicon';
wall_width=channel_pitch-channel_width;
first_wall=25; 





%% DISCRETIZING PARAMETERS OF THE CHIP- BASED ON CHANNEL DIMENSIONS
w=200;


if (channel && channel_type==mc4rm && (first_wall~=wall_width))
    l_set=[];
    l_begin=first_wall;
    l_last=l_begin;
    

    l_cur=0;
    n_l=0;
    channel_assignment_flag=1;
    n_channels=0;
    
    while(l_cur<(L-l_last))
        n_l=n_l+1;
        if n_l==1
            l_set=[l_set l_begin];
            l_cur=l_cur+l_begin;
        else
            if channel_assignment_flag
                l_set=[l_set channel_width];
                l_cur=l_cur+channel_width;
                n_channels=n_channels+1;
                channel_assignment_flag=0;
            else
                l_set=[l_set wall_width];
                l_cur=l_cur+wall_width;
                channel_assignment_flag=1;
            end
        end
    end
    if channel_assignment_flag~=0 % the last thing to be assigned before the last wall must be a channel
        disp('Error! You have not assigned channel widths and pitches properly!!');
        simulation_terminated_flag=1;
        break;
    end
    
    l_set=[l_set l_last];
    n_l=n_l+1;
    n_w=round(W/w);
else
    n_l=round(L/l);
    n_w=round(W/w);
    n_channels=(n_l-1)/2;
    if n_channels~=round(n_channels)
        disp('Error! You have not assigned channel widths and pitches properly!!');
        simulation_terminated_flag=1;
        break;
    end
    l_set=ones(1,n_l)*l;
end

Gin=700; %mass flux velocity
FM_in_set=Gin*ones(1,n_channels)*1e-12*channel_width*channel_height; %mass flow rate kg/s- derived from mass flux in kg/m2s assumed to be same for every channel layer



%% CREATING THE STACK

layer_info=[];  %Save the thickness of each physical layer
height=1;
count_h=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% EXAMPLES FOR DIFFERENT KINDS OF LAYERS %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%% NORMAL LAYER %%%%%%%%%%%%%%%%%%%%%%%%%
% layer_info_cur=struct('type','metal','height',metal_height);
% layer_info=[layer_info; layer_info_cur];
% count_h=count_h+1;


%%%%%%%%%%%%%%%%%%%%%%%%% SOURCE LAYER %%%%%%%%%%%%%%%%%%%%%%%%%
% layer_info_cur=struct('type','silicon','height',si_height_active);
% layer_info=[layer_info; layer_info_cur];
% tot_source_layers=[tot_source_layers count_h];
% tot_via_layers=[tot_via_layers count_h];
% floorplan_set=[floorplan_set; struct('name','L2')];
% die_set=[die_set; struct('name','Memory 1')];
% count_h=count_h+1;


%%%%%%%%%%%%%%%%%%%%%%%%% WATER LAYER %%%%%%%%%%%%%%%%%%%%%%%%%%
% add_channel_to_stack

%%%%%%%%%%%%%%%%%%%%%%%%%  LAYER WITH VIAS %%%%%%%%%%%%%%%%%%%%%%%%%%
% layer_info_cur=struct('type','silicon','height',si_height_bulk);
% layer_info=[layer_info; layer_info_cur];
% tot_via_layers=[tot_via_layers count_h];
% count_h=count_h+1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



si_height_bulk=50;
si_height_active=1;
metal_height=6;
glue_height=2;
tim_height=50;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







%%%%%%%%%%%%%%%%%%%%%%%%% LAYER 01- SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%

layer_info_cur=struct('type','silicon','height',2);
layer_info=[layer_info; layer_info_cur];
tot_source_layers=[tot_source_layers count_h];
floorplan_set=[floorplan_set; struct('name','background')];
die_set=[die_set; struct('name','layer_01')];
count_h=count_h+1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%% LAYER 02- SILICON %%%%%%%%%%%%%%%%%%%%%%%%%
layer_info_cur=struct('type','silicon','height',48);
layer_info=[layer_info; layer_info_cur];
count_h=count_h+1;



%%%%%%%%%%%%%%%%%%%%%%%%% LAYER 04- EVAPORATOR %%%%%%%%%%%%%%%%%%%%%%%%%%
add_channel_to_stack;

%%%%%%%%%%%%%%%%%%%%%%%%% LAYER 05- SILICON %%%%%%%%%%%%%%%%%%%%%%%%%
layer_info_cur=struct('type','silicon','height',48);
layer_info=[layer_info; layer_info_cur];
count_h=count_h+1;

%%%%%%%%%%%%%%%%%%%%%%%%% LAYER 06- SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%

layer_info_cur=struct('type','silicon','height',2);
layer_info=[layer_info; layer_info_cur];
tot_source_layers=[tot_source_layers count_h];
floorplan_set=[floorplan_set; struct('name','background')];
die_set=[die_set; struct('name','layer_02')];
count_h=count_h+1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% DEFINING THE ANALYSIS TYPE

analysis_type=steady;
error_tolerance=1e-6;

%the following numbers are ignored in case of steady state simulation
slot_time=2e-3;
input_rise_time=1e-4;
h_step=.1e-3;



%% PRINTING OUTPUT

%%% PRINTING OUTPUTS IN MATLAB FIGURES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% OUTPUT AT A PARTICULAR NODE GIVEN BY THE {COL,ROW,LAYER} SPECIFICATION --> LEAVE EMPTY IF NOT NEEDED
transient_out_node=[round(n_l/2) round(n_w/2) tot_source_layers(1)];

%

% LIST OF DIES, IDENTIFIED BY THEIR NAMES --> LEAVE EMPTY IF NOT NEEDED
snapshot_die=[struct('name', 'layer_01')
               struct('name', 'layer_02')]; 
%struct('name', 'layer_01')]; 
%               

% LIST OF LAYERS --> LEAVE EMPTY IF NOT NEEDED
snapshot_layer=[];
% snapshot_layer=[2
%                 4];

%location of cut in the die to take a vertical snapshot --> LEAVE EMPTY IF NOT NEEDED
vertical_snapshot=[]; %round(n_l/2)];

%LIST OF DIES, IDENTIFIED BY THEIR NAMES
powermap_die=[]; %struct('name','layer_01')];

%%% PRINTING OUTPUTS IN FILES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
print_output_files=TRUE;

% LIST OF DIES FOR WHICH YOU NEED TRANSIENT TEMP INFO FOR EACH FLP ELEMENT AND THE FILE NAME WHERE YOU WANT TO PRINT THE TEMPERATURES
transient_flp_elements_die=[]; 
%struct('name','CORE_MEMORY','filename','Coskun_2D_CoreL2_OnChipMC.txt')];

% LIST OF DIES FOR WHICH YOU NEED SNAPSHOTS IN A FILE
snapshot_die_on_file=[];
% 
% snapshot_die_on_file=[struct('name','CORE_MEMORY','filename','Coskun_2D_CoreL2_OnChipMC_SNAPSHOT.txt')];



%% PRINTING MATRICES FOR CHECKING- A=G+C/h IN CASE OF TRANSIENT AND A=G IN CASE OF STEADY STATE
PRINT_MATRICES=FALSE;




%% PRINTING INPUT FILES FOR THE C VERSION

PRINT_STK_FLP=FALSE;
stack_file_name='niagara_big.stk';


%% SOME PROCESSING OF THE INPUTS - DON'T REMOVE THIS!!

n_channel_layers=length(tot_channel_layers);

if analysis_type==steady
    slot_time=0;
    input_rise_time=0;
    h_step=0;
end

if analysis_type==transient
    FM_set=zeros(n_channels*n_channel_layers,n_w);
end


L_size=L;
n_layers_tot=count_h-1;
n_materials=size(material_set,1);

simulation_terminated_flag=0;



if tsv
    tsv_ratio=(tsv_area*n_tsv)/(L*W);
    
    count_mat_tsv=-1;
    for count_mat=1:n_materials
        if strcmp(tsv_material,material_set(count_mat).name)
            count_mat_tsv=count_mat;
            break;
        end
    end
    if count_mat_tsv==-1
        disp('TSV material not found');
        tsv_material
        simulation_terminated_flag=1;
        break;
    end
end

if channel
    count_mat_wall=-1;
    for count_mat=1:n_materials
        if strcmp(wall_material,material_set(count_mat).name)
            count_mat_wall=count_mat;
            break;
        end
    end
    if count_mat_wall==-1
        disp('wall material not found');
        wall_material
        simulation_terminated_flag=1;
        break;
    end
end
            
            
for count_layer=1:n_layers_tot
   mat_cur= layer_info(count_layer).type;
   count_mat_cur=-1;
            
    for count_mat=1:n_materials
        if strcmp(mat_cur,material_set(count_mat).name)
            count_mat_cur=count_mat;
            break;
        elseif strcmp(mat_cur,'channel')
            count_mat_cur=0;
            channel_flag=TRUE;
            break;
        elseif strcmp(mat_cur,'wall')
            count_mat_cur=0;
            wall_flag=TRUE;
            break;
        elseif strcmp(mat_cur,'virtual_wall')
            count_mat_cur=0;
            virtual_wall_flag=TRUE;
            break;
        end
        
    end
    
    if count_mat_cur==-1;
        disp('material not found');
        mat_cur
        simulation_terminated_flag=1;
        break;
    end

end

if air_heat_sink
    n_layers_tot=n_layers_tot+2;
    get_heat_sink_parameters;
end

if simulation_terminated_flag
    break;
end

